/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-VierConnects-core.
 *
 * ipspiel24-VierConnects-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-VierConnects-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel24-VierConnects-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel24.VierConnects.core.domain;

import java.util.List;

import de.fhdw.gaming.core.domain.Game;
import de.fhdw.gaming.core.domain.GameBuilder;
import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.core.domain.Observer;
import de.fhdw.gaming.ipspiel24.VierConnects.core.moves.VierConnectsMove;

/**
 * A builder which allows to create a Vier Connects game.
 */
public interface VierConnectsGameBuilder extends GameBuilder {

    /**
     * The default number of rows of a Vier Connects board.
     */
    int DEFAULT_NR_OF_ROWS = 7;

    /**
     * The default number of columns of a Vier Connects board.
     */
    int DEFAULT_NR_OF_COLUMNS = 8;

    /**
     * Creates an {@link VierConnectsPlayerBuilder} which allows to create and add a player to the game
     * together with her
     * strategy.
     */
    VierConnectsPlayerBuilder createPlayerBuilder();

    /**
     * Adds a player and her corresponding strategy.
     *
     * @param player   The player.
     * @param strategy The player's strategy.
     * @throws GameException if adding the player is not allowed by the rules of the game.
     */
    VierConnectsGameBuilder addPlayer(VierConnectsPlayer player, VierConnectsStrategy strategy) throws GameException;

    /**
     * Changes the number of rows (and columns) of the board.
     * <p>
     * If this operation is called multiple times, only the last board size will be retained. If not called, the default
     * size of {@link #DEFAULT_BOARD_SIZE} rows and columns is used.
     *
     * @param newNrOfRows    The new number of rows of the board.
     * @param newNrOfColumns The new number of columns of the board.
     * @return {@code this}
     */
    VierConnectsGameBuilder changeBoardSize(int newNrOfRows, int newNrOfColumns);

    @Override
    VierConnectsGameBuilder addObservers(List<Observer> newObservers);

    @Override
    Game<VierConnectsPlayer, VierConnectsState, VierConnectsMove, VierConnectsStrategy> build(int id)
            throws GameException, InterruptedException;
}
